/*
 *      MessageID-Finder for Mozilla
 *
 *  Markus Hossner - markushossner@gmx.de
 *
 */

const pmpB = 0;
const pmpN = 1;
const pmpS = 2;

var prefRead = false;
var accountURIs = new Array();
var gMIDFinderStringBundle;
var msgWindow = Components.classes["@mozilla.org/messenger/msgwindow;1"].createInstance(Components.interfaces.nsIMsgWindow);

/*
 *   Finds message via messageid
 */
function MessageIdFinder(strhostname,strport,strmessageid)
{
  var openmsginnewwin   = pmpReadPref('extensions.messageidfinder.openmsginnewwin',pmpB);
  var defaultnewsserver = pmpReadPref('extensions.messageidfinder.defaultnewsserver',pmpS);

  var strMsgIdNotFound  = MIDFinderGetString("MsgIdNotFound");

  // *** If strmessageid is empty, get messageid ****
  if (strmessageid == "")
  {
    switch(document.firstChild.id)
    {
      // MessageID-Finder called within source view
      case "main-window":
        strmessageid = String(document.getElementById('content').contentWindow.getSelection());
        openmsginnewwin = true;
      break;

      // MessageID-Finder called within msgWindow
      case "messengerWindow":
        var source = document.popupNode.localName;
        switch(source)
        {
          // MessageID-Finder called within message pane on selection
          case "PRE":
          case "DIV":
            source = document.getElementById('messagepane');
            strmessageid = String(source.contentWindow.getSelection());
          break;

          // MessageID-Finder called within message pane on link
          case "A":
            var linkText = gContextMenu.linkText();
            linkText = linkText.replace(/^<|>$/g, "");

            // Workaround for Bug 89939
            if (linkText.search(/news:[^\/]/) == 0) strmessageid  = linkText;
            else strmessageid = gContextMenu.linkURL();
          break;

          // MessageID-Finder called within message header
          case "mail-emailaddress":
            strmessageid = document.popupNode.getAttribute("sMID");
          break;

          default:
            strmessageid = "";
          break;
        }
      break;

      // MessageID-Finder called within Menulist of PrefDialog
      case "messageidfinderprefdialog":
      // MessageID-Finder called within Menulist of InputDialog
      case "midinputdialog":
      default:
        strmessageid = "";
      break;
    }
  }
  // ************************************************


  // *** Extract messageid ***
  if (strmessageid != "") strmessageid = ExtractMessageID(strmessageid);


  // *** Look for messageid ***
  var strsrc = "";
  if (strmessageid != "")
  {
    switch(strhostname)
    {

      // Look for MID locally
      case "local":
        var msgHdr = OpenMessageByMessageID(strmessageid);
        if (!msgHdr)
        {
          if (defaultnewsserver == "none" || defaultnewsserver == null)
          {
            alert(strMsgIdNotFound);
          }
          else if (defaultnewsserver.substring(0,6) == "google")
          {
            MessageIdFinder(defaultnewsserver,"",strmessageid);
          }
          else
          {
            strhostname = pmpReadPref("mail.server." + defaultnewsserver + ".hostname",pmpS);
            strport     = pmpReadPref("mail.server." + defaultnewsserver + ".port", pmpN);
            MessageIdFinder(strhostname,strport,strmessageid);
          }
        }
      break;

      // Look for MID with googl.com / google.de
      case "google.com":
      case "google.de":
         strsrc = 'http://groups.' + strhostname + '/groups?selm=' + escape(strmessageid) + '&rnum=1';
      break;

      // Look for MID with newsserver
      default:
         strsrc = 'news://' + strhostname + ':' + strport +'/'+ escape(strmessageid);
      break;

    }
  }
  // **************************


  // *** Open message ****
  if (strsrc.substring(0,4) == "news")
  {
    if (openmsginnewwin)
    {
      launchBrowser(strsrc);
    }
    else
    {
      var collapsedHeaderViewNode, expandedHeaderViewNode;

      if (document.firstChild.id == "messengerWindow")
      {
        collapsedHeaderViewNode = window.document.getElementById("collapsedHeaderView");
        expandedHeaderViewNode  = window.document.getElementById("expandedHeaderView");
        window.content.location = strsrc;
      }
      else
      {
        collapsedHeaderViewNode = window.opener.document.getElementById("collapsedHeaderView");
        expandedHeaderViewNode  = window.opener.document.getElementById("expandedHeaderView");
        window.opener.content.location = strsrc;
      }

      if (collapsedHeaderViewNode) collapsedHeaderViewNode.collapsed = true;
      if (expandedHeaderViewNode) expandedHeaderViewNode.collapsed = true;
    }

  }
  else if (strsrc.substring(0,4) == "http")
  {
    launchBrowser(strsrc);
  }
  // *********************
}


function ExtractMessageID(strmessageid)
{
  strmessageid = strmessageid.replace(/^<|>$/g, "");

  var midParts = strmessageid.split(":");
  switch (midParts.length)
  {
    case 1:
      strmessageid = strmessageid;
    break;

    case 2:
      if (midParts[0] == "news" || midParts[0] == "nntp")
      {
        strmessageid = midParts[1].replace(/^\/\/.*?\//,"");
      }
      else if (midParts[0] == "mailto")
      {
        strmessageid = midParts[1];
      }
    break;

    case 3:
      strmessageid = midParts[2].replace(/^.+?\//,"");
    break;

    default:
      strmessageid = "";
    break;
  }

  return strmessageid;
}


/*
 *   Opens and selects a message by messageid
 */
function OpenMessageByMessageID(messageId)
{
  var openmsginnewwin = pmpReadPref('extensions.messageidfinder.openmsginnewwin',pmpB);
  window.setCursor("wait");

  var msgHdr = SearchMessageLocally(messageId);
  if (msgHdr)
  {
    var folderUri = msgHdr.folder.URI;

    if (openmsginnewwin)
    {
      var folder = GetMsgFolderFromUri(folderUri);
      var messageUri = folder.getUriForMsg(msgHdr);

      window.openDialog("chrome://messenger/content/messageWindow.xul", "_blank", "all,chrome,dialog=no,status,toolbar", messageUri, folderUri, null);
    }
    else
    {
      var tree = {};
      var msgDB = {};
      var length = {};
      var wintype = {};
      var msgDBView = {};
      var indicesArray = {};

      if (document.firstChild.id == "messengerWindow")
      {
        wintype = window.document.firstChild.getAttribute("windowtype");
        msgWindow.SetDOMWindow(window);

        folder = GetMsgFolderFromUri(folderUri);
        msgDB = folder.getMsgDatabase(msgWindow);
        msgDB.MarkHdrRead(msgHdr, false, null);
        msgWindow.SelectFolder(folderUri);

        msgDBView = window.gDBView;
        if (wintype != "mail:messageWindow") tree = window.GetThreadTree();
      }
      else
      {
        wintype = window.opener.document.firstChild.getAttribute("windowtype");
        msgWindow.SetDOMWindow(window.opener);

        folder = GetMsgFolderFromUri(folderUri);
        msgDB = folder.getMsgDatabase(msgWindow);
        msgDB.MarkHdrRead(msgHdr, false, null);
        msgWindow.SelectFolder(folderUri);

        msgDBView = window.opener.gDBView;
        if (wintype != "mail:messageWindow") tree = window.opener.GetThreadTree();
      }

      if (wintype != "mail:messageWindow") tree.view.selection.clearSelection();
      msgDBView.selectMsgByKey(msgHdr.messageKey);
      msgDBView.loadMessageByMsgKey(msgHdr.messageKey);
      msgDB.MarkHdrRead(msgHdr, true, null);
      msgDBView.getIndicesForSelection(indicesArray,length);
      if (wintype != "mail:messageWindow") tree.treeBoxObject.ensureRowIsVisible(indicesArray.value[0]);
    }
  }

 window.setCursor("auto");
 return msgHdr;
}


/*
 *   Searchs for messageid in local folders
 */
function SearchMessageLocally(messageId)
{
  var msgHdr = null;
  var folder = null;

  var x = 0;
  do
  {
    folder = GetMsgFolderFromUri(accountURIs[x]).rootFolder;
    msgHdr = SearchInSubFolder(folder, messageId);
    x++;
  }
  while(!msgHdr && x < accountURIs.length)

  return msgHdr;
}


/*
 *   Searchs in subfolder
 */
function SearchInSubFolder(folder, messageId)
{
  if (folder.hasSubFolders)
  {
    var subfolders = folder.GetSubFolders();
    var subfolder = null;
    var msgHdr = null;
    var msgDB = null;
    var currentFolderURI = "";

    var done = false;
    while(!done)
    {
      currentFolderURI = subfolders.currentItem().QueryInterface(Components.interfaces.nsIRDFResource).Value;
      subfolder = GetMsgFolderFromUri(currentFolderURI);

      if (currentFolderURI.substring(1,7) != "news://")  msgHdr = SearchInSubFolder(subfolder, messageId);

      if (!msgHdr)
      {
        msgDB = subfolder.getMsgDatabase(msgWindow);
        msgHdr = msgDB.getMsgHdrForMessageID(messageId);
      }

      if (msgHdr) return msgHdr;

      try {subfolders.next();}
      catch(e) {done = true;}
    }
  }
 return null;
}


/*
 *   Searchs all availible newsserver
 */
function AddNewsServers()
{
  if (prefRead == false)
  {
    var accounts = pmpReadPref("mail.accountmanager.accounts",pmpS);
    if (accounts)
    {
      var accountsArray = accounts.split(",");
      for (var x=0; x<accountsArray.length; x++)
      {
        AddNewsServerMenuItem(accountsArray[x]);
      }
      accountURIs.sort(sortAccountUris);
    }

    prefRead = true;
  }
}


/*
 *   Sortfunction for accountURIs Array
 *   Local Folder to the rear
 */
function sortAccountUris(a, b)
{
  if (a.search(/Local%20Folders/) != -1) return 1;
  else if (b.search(/Local%20Folders/) != -1) return -1;
  else if (a < b) return -1;
  else if (a > b) return 1;
  else return 0
}


/*
 *   Adds newsserver to the context menu
 */
function AddNewsServerMenuItem(account)
{
  var messageidfindermenu = document.getElementById("messageidfinder-popup");

  var server    = pmpReadPref("mail.account." + account + ".server",pmpS);
  var servertyp = pmpReadPref("mail.server."  + server +  ".type",pmpS);
  var hostname  = pmpReadPref("mail.server."  + server +  ".hostname",pmpS);
  var username  = pmpReadPref("mail.server."  + server +  ".userName",pmpS);

  var accountURI = "";
  if (servertyp == "nntp")
  {
    var servername = pmpReadPref("mail.server." + server + ".name",pmpS);
    var serverport = pmpReadPref("mail.server." + server + ".port", pmpN);
    var newsservermenuitem;

    if (servername == null) servername = hostname;
    if (serverport == null) serverport = 119;

    newsservermenuitem = document.createElement("menuitem");
    newsservermenuitem.setAttribute("label", servername);
    newsservermenuitem.setAttribute("value",server);
    newsservermenuitem.setAttribute("oncommand","MessageIdFinder('" + hostname + "', '" + String(serverport) + "','');");
    messageidfindermenu.appendChild(newsservermenuitem);

    accountURI = "news://" + escape(hostname);
  }
  else if (servertyp == "imap")
  {
    accountURI = "imap://" + escape(username) + "@" + escape(hostname);
  }
  else
  {
    accountURI = "mailbox://" + escape(username) + "@" + escape(hostname);
  }

  if (hostname != null) accountURIs.push(accountURI);
}



/*
 *   Reads the value of pmpName out of the mozilla preferences
 */
function pmpReadPref(pmpName, pmpType)
{
   var nsIPrefBranch = Components.interfaces.nsIPrefBranch;
   var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(nsIPrefBranch);
   var pmpValue = null;

   if (pref.prefHasUserValue(pmpName))
   {
     switch ( pmpType )
     {
       case pmpB:
         pmpValue = pref.getBoolPref(pmpName);
       break;
       case pmpN:
         pmpValue = pref.getIntPref(pmpName);
       break;
       case pmpS:
         pmpValue = pref.getCharPref(pmpName);
       break;
       default:
       break;
     }
   }
   return pmpValue;
}


/*
 *   Get msgfolder from Uri
 */
function GetMsgFolderFromUri(uri)
{
  var msgfolder = null;
  try
  {
    var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
    var resource = RDF.GetResource(uri);

    if (resource instanceof Components.interfaces.nsIMsgFolder && (resource.parent || resource.isServer))
      msgfolder = resource;
  }
  catch (e)
  {
    dump("<Error MID-Finder>\n");
    for (var s in e) dump(s + " = " + e[s] + "\n");
    dump("</Error MID-Finder>\n");
  }

  return msgfolder;
}


/*
 *   Get string from stringbundle
 */
function MIDFinderGetString(name)
{
  try
  {
    if (!gMIDFinderStringBundle)
    {
      var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
      gMIDFinderStringBundle = strBundleService.createBundle("chrome://messageidfinder/locale/messageidfinder.properties");
    }

    if (gMIDFinderStringBundle) return gMIDFinderStringBundle.GetStringFromName(name);
  }
  catch (e)
  {
    dump("<Error MID-Finder>\n");
    for (var s in e) dump(s + " = " + e[s] + "\n");
    dump("</Error MID-Finder>\n");
  }

  return null;
}



function launchBrowser(UrlToGoTo)
{
   if(navigator.vendor != "Thunderbird") {
     var navWindow;

     // if not, get the most recently used browser window
     try {
       var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
               .getService(Components.interfaces.nsIWindowMediator);
       navWindow = wm.getMostRecentWindow("navigator:browser");
     }
     catch (ex) { }

     if (navWindow) {
        if ("delayedOpenTab" in navWindow)
          navWindow.delayedOpenTab(UrlToGoTo);
        else if ("loadURI" in navWindow)
          navWindow.loadURI(UrlToGoTo);
        else
          navWindow._content.location.href = UrlToGoTo;
      }
      // if all else fails, open a new window
      else {

        var ass = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
        var win = ass.hiddenDOMWindow;

        win.openDialog("chrome://navigator/content/navigator.xul", "_blank", "chrome,all,dialog=no", UrlToGoTo );
     }
    }
    else
      {
        try {
             var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();
             messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
             messenger.launchExternalURL(UrlToGoTo);
        } catch (ex) {}
      }
}